<?php

use Phinx\Db\Adapter\MysqlAdapter;
use Phinx\Migration\AbstractMigration;

class AddTableSelection extends AbstractMigration
{
    /**
     * Change Method.
     *
     * Write your reversible migrations using this method.
     *
     * More information on writing migrations is available here:
     * http://docs.phinx.org/en/latest/migrations.html#the-abstractmigration-class
     *
     * The following commands can be used in this method and Phinx will
     * automatically reverse them when rolling back:
     *
     *    createTable
     *    renameTable
     *    addColumn
     *    addCustomColumn
     *    renameColumn
     *    addIndex
     *    addForeignKey
     *
     * Any other destructive changes will result in an error when trying to
     * rollback the migration.
     *
     * Remember to call "create()" or "update()" and NOT "save()" when working
     * with the Table class.
     */
    public function change()
    {
        $this
            ->table("selection", ['id' => false, 'primary_key' => ["selection_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('selection_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('selection_type', 'string', ['null' => false])
            ->addColumn('parent_id', 'integer', ['null' => true, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'selection_id', 'signed' => false])
            ->addColumn('selection_rang', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'parent_id'])
            ->addColumn('selection_date_ajout', 'datetime', ['null' => false, 'default' => 'CURRENT_TIMESTAMP', 'after' => 'selection_rang'])
            ->addColumn('selection_date_modif', 'datetime', ['null' => false, 'default' => 'CURRENT_TIMESTAMP', 'after' => 'selection_date_ajout'])
            ->addColumn('selection_image', 'string', ['null' => false, 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'selection_date_modif'])
            ->addColumn('selection_ext_id', 'string', ['null' => false, 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'selection_image'])
            ->addForeignKey('parent_id', 'selection', 'selection_id', [
                'delete' => 'SET NULL',
                'update' => 'CASCADE'
            ])
            ->addIndex(['parent_id'], ['unique' => false])
            ->addIndex(['selection_ext_id'], ['unique' => false])
            ->addIndex(['selection_rang'], ['unique' => false])
            ->create();

        $this
            ->table("selection_boutique", ['id' => false, 'primary_key' => ["selection_boutique_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('selection_boutique_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('selection_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'selection_boutique_id', 'signed' => false])
            ->addColumn('boutique_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'selection_id', 'signed' => false])
            ->addColumn('selection_boutique_statut_id', 'integer', ['null' => false, 'default' => '1', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'boutique_id'])
            ->addIndex(['selection_id', 'boutique_id'], ['unique' => true])
            ->addIndex(['selection_id'], ['unique' => false])
            ->addIndex(['boutique_id'], ['unique' => false])
            ->addIndex(['selection_boutique_statut_id'], ['unique' => false])
            ->addForeignKey('selection_id', 'selection', 'selection_id', [
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ])
            ->addForeignKey('boutique_id', 'boutique', 'boutique_id', [
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ])
            ->create();

        $this
            ->table("selection_nom", ['id' => false, 'primary_key' => ["selection_nom_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('selection_nom_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('selection_id', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'selection_nom_id', 'signed' => false])
            ->addColumn('langue_id', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'selection_id', 'signed' => false])
            ->addColumn('selection_nom_nom', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'langue_id'])
            ->addColumn('selection_nom_desc', 'text', ['null' => false, 'limit' => 65535, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'selection_nom_nom'])
            ->addForeignKey('selection_id', 'selection', 'selection_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('langue_id', 'langue', 'langue_id', [
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ])
            ->addIndex(['langue_id', 'selection_id'], ['name' => "langue_id_2", 'unique' => true])
            ->addIndex(['langue_id'], ['name' => "langue_id", 'unique' => false])
            ->addIndex(['selection_id'], ['name' => "selection_id", 'unique' => false])
            ->create();
    }
}
