<?php

/**
 * Class Adresse
 *
 * @property $pays_id
 */
class Adresse extends Model
{
    const TYPE_LIVRAISON = 1;
    const TYPE_FACTURATION = 2;

    protected $table = "adresse";

    public function __construct($id = null)
    {
        $this->leftJoin[] = array(
            'civilite_nom',
            'AND',
            array('a.civilite_id', 'cn.civilite_id'),
            array('cn.langue_id', Boutique::getInstance()->getLangue()->id)
        );

        $this->leftJoin[] = array('pays', 'a.pays_id', 'p.pays_id');

        $this->leftJoin[] = array(
            'pays_nom',
            'AND',
            array('a.pays_id', 'pn.pays_id'),
            array('pn.langue_id', Boutique::getInstance()->getLangue()->id)
        );

        $this->extras["order"] = "a.adresse_default DESC, a.adresse_libelle ASC";

        parent::__construct($id);
    }

    /**
     * Retourne l'adresse par défaut d'un client
     *
     * @param $clientId
     *
     * @return Adresse
     * @throws Exception
     */
    public function getDefault($clientId)
    {
        $addr = new Adresse();

        $addr->hydrate(
            $this->sql->getOne(
                $this->getQuery(
                    array(
                        'a.client_id' => $clientId,
                        'a.adresse_type_id' => self::TYPE_LIVRAISON
                    ))
                    ->order('a.adresse_default DESC')
                    ->build()
            )
        );
        return $addr;
    }

    /**
     * Retourne l'adresse de facturation d'un client
     *
     * @param $clientId
     *
     * @return Adresse
     * @throws Exception
     */
    public function getBilling($clientId)
    {
        $addr = new Adresse();

        $addr->hydrate(
            $this->sql->getOne(
                $this->getQuery(array('a.client_id' => $clientId, 'a.adresse_type_id' => self::TYPE_FACTURATION))->build()
            )
        );
        return $addr;
    }

    /**
     * Ajoute une nouvelle adresse
     *
     * @param array $formvars tableau contenant les données pour l'ajout de l'adresse
     *
     * @return mixed
     * @deprecated
     */
    function add($formvars = array())
    {

        if (empty($formvars['isAlreadyEscaped'])) {
            Tools::escapeDataToSql($formvars);
        }

        $sql = "INSERT INTO adresse SET";
        $sql.= " client_id = '" . $formvars['client_id'] . "'";
        $sql.= ", adresse_libelle = '" . $formvars['adresse_libelle'] . "'";
        $sql.= ", civilite_id = '" . $formvars['civilite_id'] . "'";
        $sql.= ", adresse_societe = '" . $formvars['adresse_societe'] . "'";
        $sql.= ", adresse_nom = '" . $formvars['adresse_nom'] . "'";
        $sql.= ", adresse_prenom = '" . $formvars['adresse_prenom'] . "'";
        $sql.= ", adresse_rue = '" . $formvars['adresse_rue'] . "'";
        $sql.= ", adresse_rue2 = '" . $formvars['adresse_rue2'] . "'";
        $sql.= ", adresse_cp = '" . $formvars['adresse_cp'] . "'";
        $sql.= ", adresse_ville = '" . $formvars['adresse_ville'] . "'";
        $sql.= ", adresse_digicode = '" . $formvars['adresse_digicode'] . "'";
        $sql.= ", adresse_batiment = '" . $formvars['adresse_batiment'] . "'";
        $sql.= ", adresse_etage = '" . $formvars['adresse_etage'] . "'";
        $sql.= ", adresse_quartier = '" . $formvars['adresse_quartier'] . "'";
        if (!empty($formvars['adresse_etat'])) {
            $sql .= ", adresse_etat = '" . $formvars['adresse_etat'] . "'";
        }
        $sql.= ", pays_id = '" . $formvars['pays_id'] . "'";
        $sql.= ", adresse_telephone = '" . $formvars['adresse_telephone_mobile'] . "'";

        $this->sql->query($sql);
        return $this->sql->db->insert_id;

    }

    // mise a jour d'une adresse
    function update($formvars = array()) {

        Tools::escapeDataToSql($formvars);

        $sql = "UPDATE adresse SET";
        $sql.= " civilite_id = '" . $formvars['civilite_id'] . "'";
        $sql.= ", adresse_libelle = '" . $formvars['adresse_libelle'] . "'";
        $sql.= ", adresse_societe = '" . $formvars['adresse_societe'] . "'";
        $sql.= ", adresse_nom = '" . $formvars['adresse_nom'] . "'";
        $sql.= ", adresse_prenom = '" . $formvars['adresse_prenom'] . "'";
        $sql.= ", adresse_rue = '" . $formvars['adresse_rue'] . "'";
        $sql.= ", adresse_rue2 = '" . $formvars['adresse_rue2'] . "'";
        $sql.= ", adresse_cp = '" . $formvars['adresse_cp'] . "'";
        $sql.= ", adresse_ville = '" . $formvars['adresse_ville'] . "'";
        $sql.= ", adresse_digicode = '" . $formvars['adresse_digicode'] . "'";
        $sql.= ", adresse_batiment = '" . $formvars['adresse_batiment'] . "'";
        $sql.= ", adresse_etage = '" . $formvars['adresse_etage'] . "'";
        $sql.= ", adresse_quartier = '" . $formvars['adresse_quartier'] . "'";
        $sql.= ", adresse_telephone = '" . $formvars['adresse_telephone'] . "'";
        if(!empty($formvars['adresse_etat']))
            $sql.= ", adresse_etat = '" . $formvars['adresse_etat'] . "'";
        $sql.= ", pays_id = '" . $formvars['pays_id'] . "'";
        $sql.= " WHERE adresse_id = '" . $formvars['adresse_id'] . "'";
        $sql.= " AND client_id = '" . $formvars['client_id'] . "'";
        $this->sql->query($sql);

        return $formvars['adresse_id'];

    }


    /**
     * @param array $formvars
     * @throws DediException
     */
    function delete($formvars = array())
    {
        $sql = "DELETE FROM adresse 
                WHERE adresse_id = '" . $formvars['adresse_id'] . "' 
                AND client_id = '" . $formvars['client_id'] . "'
                AND adresse_type_id != " . self::TYPE_FACTURATION;
        $this->sql->query($sql);
    }

    /**
     * On clean les adresses en ajoutant les manquantes
     * Le client doit avoir :
     * - une adresse type facturation
     * - au moins une adresse default en type livraison (si pas d'adresse livraison, pas de default)
     * @param $idClient
     */
    public function cleanAddress($idClient)
    {
        $arrAdresses = $this->get(array('client_id' => $idClient));

        if (!empty($arrAdresses)) {
            $addrBilling = array();
            $addrShippingDefault = array();
            $addrFree = array();

            foreach ($arrAdresses as $a) {
                if ($a['adresse_type_id'] == self::TYPE_FACTURATION) {
                    $addrBilling = $a;
                } elseif (!empty($a['adresse_default'])) {
                    $addrShippingDefault = $a;
                } else {
                    $addrFree[] = $a;
                }
            }

            if (empty($addrBilling) || empty($addrShippingDefault)) {

                if(empty($addrShippingDefault) && !empty($addrFree)) {
                    $arrNewAdress = $addrFree[0];
                    $arrNewAdress['adresse_default'] = 1;
                    $arrNewAdress['adresse_id'] = $this->sql->save('adresse', $arrNewAdress);
                    $addrShippingDefault = $arrNewAdress;
                }

                if (empty($addrBilling) && !empty($addrShippingDefault)) {
                    $arrNewAdress = $addrShippingDefault;
                    unset($arrNewAdress['adresse_id']);
                    $arrNewAdress['adresse_default'] = 0;
                    $arrNewAdress['adresse_type_id'] = Adresse::TYPE_FACTURATION;
                    $this->sql->save('adresse', $arrNewAdress);
                }
            }
        }
    }
}