<?php

use Tree\Tree;
use Tree\Node;

use Symfony\Component\Routing\Exception\InvalidParameterException;

/**
 * Class Categorie
 *
 * @property integer $categorie_id
 * @property integer $categorie_niveau
 */
final class Categorie extends Model
{
    protected $table = "categorie";

    static protected $tree = null;

    public function __construct($id = null)
    {
        $this->innerJoin[] = [
            "categorie_boutique cb",
            "AND",
            ["cb.categorie_id", "c.categorie_id"],
            ["cb.boutique_id", Boutique::getInstance()->id]
        ];

        parent::__construct($id);
    }

    function get($formvars = array(), $champs = "")
    {

        $lgId = Boutique::getInstance()->getLangue()->id;

        $sql = "SELECT *, c.parent_id, c.categorie_id, c.categorie_image, cnpar.categorie_nom_nom as parent_nom_nom";
        if (!empty($formvars['products']) || !empty($formvars['nbprod'])) {
            $sql .= ", COUNT(pc.produit_id) as nbProd";
        }

        $sql .= " FROM (categorie c)";
        if (empty($formvars['no_boutique'])) {
            $sql .= " INNER JOIN categorie_boutique cb ON (c.categorie_id = cb.categorie_id AND cb.boutique_id = '" . Boutique::getInstance()->id . "')";
        }
        $sql .= " LEFT JOIN categorie_nom cnpar ON (cnpar.categorie_id = c.parent_id AND cnpar.langue_id='" . $lgId . "')";
        $sql .= " LEFT JOIN categorie_nom cn ON (c.categorie_id=cn.categorie_id AND cn.langue_id='" . $lgId . "')";
        if (!empty($formvars['products']) || !empty($formvars['nbprod'])) {
            $sql .= " LEFT JOIN produit_categorie pc ON (pc.categorie_id = c.categorie_id)";
        }

        $sql .= " WHERE 1";
        if (empty($formvars['no_boutique'])) {
            $sql .= " AND cb.categorie_boutique_statut_id = '1'";
        }
        if (!empty($formvars['categorie_id']))
            $sql .= " AND c.categorie_id = '" . $formvars['categorie_id'] . "'";
        if (!empty($formvars['categorie_id_not_in']))
            $sql .= " AND c.categorie_id NOT IN (" . $formvars['categorie_id_not_in'] . ")";
        if (!empty($formvars['parent_id']))
            $sql .= " AND c.parent_id = '" . $formvars['parent_id'] . "'";
        if (!empty($formvars['only_parent']))
            $sql .= " AND c.parent_id IS NULL";
        if (!empty($formvars['no_parent']))
            $sql .= " AND c.parent_id IS NOT NULL";

        if(isset($formvars['categorie_niveau'])){
            $sql.= " AND c.categorie_niveau = {$formvars['categorie_niveau']}";
        }

        if(!empty($formvars['not']))

            $sql .= " GROUP BY c.categorie_id";

        if (!empty($formvars['products'])) {
            $sql .= " HAVING nbProd > 0";
        }

        if (empty($formvars['order']))
            $formvars['order'] = '';

        switch ($formvars['order']) {
            case 'nom':
                $sql .= " ORDER BY cn.categorie_nom_nom";
                break;
            case 'nbprod':
                $sql .= " ORDER BY nbProd ASC";
                break;
            case 'rang_desc':
                $sql .= " ORDER BY c.categorie_rang DESC";
                break;
            default:
                $sql .= " ORDER BY c.categorie_rang";
        }

        if (empty($champs)) {
            $this->sql->query($sql, SiteSQL::SQL_ALL, SiteSQL::SQL_ASSOC);
            return $this->sql->record;
        } else {
            $this->sql->query($sql, SiteSQL::SQL_INIT, SiteSQL::SQL_ASSOC);
            return $this->sql->record[$champs];
        }
    }

    /**
     * @return string
     */
    public function resolvePrimaryKey()
    {
        return "categorie_id";
    }
}