<?php

namespace Controller;

use CategorieSpec;
use DediException;
use Exception;
use Gamme;
use Listing;
use Breadcrumb;
use Boutique;
use Categorie;
use Marque;
use SmartyException;
use Texte;

use Symfony\Component\Routing\Annotation\Route;
use Tools;

/**
 * Class ListingController
 * @package Controller
 */
class ListingController extends Controller
{
    /**
     * @Route(
     *      path         = "/recherche/{features}",
     *      name         = "recherche",
     *      defaults     = { "features" = null },
     *      requirements = { "features" = "[a-zA-Z0-9-_,]+" }
     * )
     */
    public function search()
    {
        $texte = new Texte();
        $listing = new Listing();
        $motclef = $this->request->get('motclef');
        $listing->setSearch($motclef);

        $this->template->assign('titre', $motclef);
        $this->template->assign(
            'desc',
            $texte->getOneOrCreate(
                'recherche_mots_cles',
                "Recherche de mots-clés"
            ) . ' &laquo; <h2>' . $motclef . '</h2> &raquo;'
        );

        Breadcrumb::getInstance()->add($texte->getOneOrCreate('recherche', 'Recherche'))
            ->add($motclef);

        return $this->show($listing);
    }

    /**
     * @Route(
     *      path         = "/nos-marques/{marque_nom}-{marque_id}/{features}",
     *      name         = "marque",
     *      defaults     = { "features" = null},
     *      requirements = { "marque_nom" = "[a-zA-Z0-9-_]+", "marque_id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+"}
     * )
     */
    public function marque()
    {
        $listing = new Listing();

        $marque = new Marque($this->request->get('marque_id'));
        $listing->addMarque($marque);

        $this->template->assign('titre', $marque->marque_nom);
        $this->template->assign('desc', $marque->marque_nom_desc);

        Breadcrumb::getInstance()->add(
            $marque->marque_nom,
            $this->generateUrl(
                'marque',
                ['marque_nom' => Tools::parseURL($marque->marque_nom), 'marque_id' => $marque->marque_id]
            )
        );

        return $this->show($listing);
    }


    /**
     * @Route(
     *      path         = "/selection/{gamme_nom}-{id}/{features}",
     *      name         = "liste_gamme_1",
     *      defaults     = { "features" = null },
     *      requirements = { "gamme_nom" = "[a-zA-Z0-9-_]+", "id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+" }
     * )
     *
     * @return mixed
     * @throws SmartyException
     */
    public function gamme()
    {
        return $this->showGamme();
    }

    /**
     * @Route(
     *      path         = "/selection/{parent_nom}/{gamme_nom}-{id}/{features}",
     *      name         = "liste_gamme_2",
     *      defaults     = { "features" = null },
     *      requirements = { "parent_nom" = "[a-zA-Z0-9-_]+", "gamme_nom" = "[a-zA-Z0-9-_]+", "id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+" }
     * )
     *
     * @return mixed
     * @throws SmartyException
     */
    public function subGamme()
    {
        return $this->showGamme();
    }

    /**
     * @Route(
     *      path         = "/{grand_parent_nom}/{parent_nom}/{gamme_nom}-{id}/{features}",
     *      name         = "liste_gamme_3",
     *      defaults     = { "features" = null },
     *      requirements = { "grand_parent_nom" = "[a-zA-Z0-9-_]+", "parent_nom" = "[a-zA-Z0-9-_]+", "gamme_nom" = "[a-zA-Z0-9-_]+", "id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+" }
     * )
     *
     * @return mixed
     * @throws SmartyException
     */
    public function subSubGamme()
    {
        return $this->showGamme();
    }

    /**
     * @return mixed|\Symfony\Component\HttpFoundation\RedirectResponse
     * @throws SmartyException
     * @throws Exception
     */
    private function showGamme()
    {
        $routeName = $this->request->attributes->get('_route');
        $listing = new Listing();
        $gamme = new Gamme($this->request->get('id'));

        if(!$gamme->isLoaded()){ // La gamme n'éxiste pas, ou pas dans categorie_boutique
            return $this->redirect($this->generateUrl("home"));
        }

        $branch = $gamme->getBranch($gamme->gamme_id);
        $validUrl = $gamme->generateUrlFromBranch($branch);

        $currentUrlparams = array_reduce(
            ['id', 'gamme_nom', 'parent_nom', 'grand_parent_nom'],
            function ($params, $key) {
                $value = $this->request->get($key);
                if (!empty($value)) {
                    $params[$key] = $value;
                }
                return $params;
            },
            []
        );

        $currentGammeUrl = $this->generateUrl($routeName, $currentUrlparams);
        if ($currentGammeUrl !== $validUrl) {
            if ($this->request->getQueryString() != null) {
                $validUrl .= '?' . $this->request->getQueryString();
            }
            return $this->redirect($validUrl, 301);
        }

        $listing->addSelection($gamme);
        $this->template->assign('categorie', $gamme->toArray());
        $this->template->assign("habillage", Listing\Dto\Habillage::fromSelection($gamme));

        $gamme->buildBreadcrumb($gamme->gamme_id);

        return $this->show($listing);
    }

    /**
     * @Route(
     *      path         = "/{categorie_nom}-{id}/{features}",
     *      name         = "liste_1",
     *      defaults     = { "features" = null },
     *      requirements = { "categorie_nom" = "[a-zA-Z0-9-_]+", "id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+" }
     * )
     *
     * @return mixed
     * @throws DediException
     */
    public function category()
    {
        return $this->showCategory();
    }

    /**
     * @Route(
     *      path         = "/{parent_nom}/{categorie_nom}-{id}/{features}",
     *      name         = "liste_2",
     *      defaults     = { "features" = null },
     *      requirements = { "parent_nom" = "[a-zA-Z0-9-_]+", "categorie_nom" = "[a-zA-Z0-9-_]+", "id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+" }
     * )
     *
     * @return mixed
     * @throws DediException
     */
    public function subcategory()
    {
        return $this->showCategory();
    }

    /**
     * @Route(
     *      path         = "/{grand_parent_nom}/{parent_nom}/{categorie_nom}-{id}/{features}",
     *      name         = "liste_3",
     *      defaults     = { "features" = null },
     *      requirements = { "grand_parent_nom" = "[a-zA-Z0-9-_]+", "parent_nom" = "[a-zA-Z0-9-_]+", "categorie_nom" = "[a-zA-Z0-9-_]+", "id" = "[0-9]+", "features" = "[a-zA-Z0-9-_,]+" }
     * )
     *
     * @return mixed
     * @throws DediException
     * @throws Exception
     */
    public function subsubcategory()
    {
        return $this->showCategory();
    }

    /**
     * Permet d'afficher un listing de produit via les catégories
     *
     * @return mixed
     * @throws DediException
     * @throws Exception
     */
    protected function showCategory()
    {
        $routeName = $this->request->attributes->get('_route');
        $listing = new Listing();

        $categorie = new Categorie($this->request->get('id'));
        $categorieSpec = new CategorieSpec($this->request->attributes->get('id'));

        if(!$categorie->isLoaded()){ // La gamme n'éxiste pas, ou pas dans categorie_boutique
            return $this->redirect($this->generateUrl("home"));
        }

        $branch = $categorieSpec->getBranch($categorie->categorie_id);
        $validUrl = $categorieSpec->generateUrlFromBranch($branch);

        $currentUrlparams = array_reduce(
            ['id', 'categorie_nom', 'parent_nom', 'grand_parent_nom'],
            function ($params, $key) {
                $value = $this->request->get($key);
                if (!empty($value)) {
                    $params[$key] = $value;
                }
                return $params;
            },
            []
        );

        $currentCategorieUrl = $this->generateUrl($routeName, $currentUrlparams);
        if ($currentCategorieUrl !== $validUrl) {
            if ($this->request->getQueryString() != null) {
                $validUrl .= '?' . $this->request->getQueryString();
            }
            return $this->redirect($validUrl, 301);
        }

        $listing->addCategorie($categorie);
        $this->template->assign('categorie', $categorie);

        $this->template->assign('titre', $categorie->categorie_nom_nom);
        $this->template->assign('desc', $categorie->categorie_nom_desc);
        $this->template->assign('image_listing', $categorie->categorie_image);
        $this->template->assign("habillage", Listing\Dto\Habillage::fromSelection($categorieSpec));

        $categorieSpec->buildBreadcrumb($categorie->categorie_id);

        return $this->show($listing);
    }

    /**
     * Permet d'afficher un listing produit en fonciton du param Listing passé.
     * Permet de récupérer les produits a afficher / calculer les cara a afficher
     *
     * @param Listing $listing
     *
     * @return mixed
     *
     * @throws \DediException
     * @throws SmartyException
     */
    public function show(Listing $listing)
    {

        $config = Boutique::getInstance()->getConfig();
        // on injecte la request dans l'objet listing pour générer les urls
        $listing->setRequest($this->request);

        $attributes = $this->request->attributes->all();
        $urlPage = $this->generateUrl(
            $attributes['_route'],
            array_diff_key($attributes, array('_route' => '', 'features' => ''))
        );
        $this->template->assign('urlPage', $urlPage);

        if ($this->request->get('features') != false) {

            $listing->setCaracteristiqueUrl($this->request->get('features'));

            $arrCaraValeur = $listing->getCaracteristiqueSelect();
            $titre = $this->template->getTemplateVars('titre');
            foreach ($arrCaraValeur as $caraVal) {
                $titre.= " " . $caraVal['caracteristique_valeur_nom_nom'];
            }
            $this->template->assign('titre', $titre);

            if (count($arrCaraValeur) >= $config['listing']['max_cara']) {
                $this->template->assign('metaRobots', 'noindex, nofollow');
            }
        }

        if ($this->request->get('page') != false) {
            $listing->setPage($this->request->get('page'));
        }

        if ($this->request->get('limit') != false) {
            $listing->setLimit($this->request->get('limit'));
        }

        if ($this->request->get('order') != false) {
            $listing->setOrder($this->request->get('order'));
        }

        if ($this->request->get('prix_min') != null && $this->request->get('prix_max') != null) {
            $param = array($this->request->get('prix_min'), $this->request->get('prix_max'));
            $paramUrl = array(
                'prix_min' => $this->request->get('prix_min'),
                'prix_max' => $this->request->get('prix_max')
            );
            $listing->setPrix($param);
            $this->template->assign('prix_min', $this->request->get('prix_min'));
            $this->template->assign('prix_max', $this->request->get('prix_max'));
            $this->template->assign('urlWithPrice', true);
            $this->template->assign('urlPrice', http_build_query($paramUrl));
        }

        $arrProd = $listing->getProductsToDisplay();

        $this->template->assign('arrProd', $arrProd);
        $this->template->assign('countProduct', $listing->getCountProducts());

        // filtres prix
        $listing->setPrice($listing->getProducts());
        if (!empty($arrProd)) {
            $this->template->assign('lepluspetitprix', $listing->getPrixMin());
            $this->template->assign('leplusgrandprix', $listing->getPrixMax());
            $this->template->assign('baseUrl', ($this->request->get('features') ?
                $this->request->get('features') :
                $urlPage
            ));
        }

        $this->template->assign('arrCaracteristique', $listing->getCaracteristique());

        $this->template->assign('arrCritere', $listing->getCaracteristiqueSelect());

        $this->template->assign('arrChildCateg', $listing->getChildCategories());

        $pagination = $listing->getPagination();
        $this->template->assign('pagination', $pagination);

        $ref = $listing->getReferencement();
        $this->template->assign('ref', $ref);

        return $this->render('listing.tpl');
    }
}
