<?php

namespace Controller;

use Breadcrumb;
use Boutique;
use Categorie;
use Produit;
use Texte;
use Attribut;
use Tools;

use Symfony\Component\Routing\Annotation\Route;

/**
 * Class ProductController
 * @package Controller
 */
class ProductController extends Controller
{

    /**
     * @Route(
     *      path         = "/{produit_nom}-{id}.html",
     *      name         = "produit",
     *      requirements = { "produit_nom" = "[a-zA-Z0-9-_]+", "id" = "[0-9]+" }
     * )
     *
     * @return mixed
     * @throws \DediException
     * @throws \Exception
     * @throws \SmartyException
     */
    public function show()
    {
        $categorieSpec = new \CategorieSpec();
        $produit = new Produit();
        $attribut = new Attribut();
        $caracteristique = new \Caracteristique();
        $txt = new Texte();
        $bd = Breadcrumb::getInstance();

        $prod = $produit->getOne(array('produit_id' => $this->request->get('id')));

        if (empty($prod)) {
            return $this->redirect($this->generateUrl('home'));
        }

        // Test bonne url
        $goodUrl = $this->generateUrl(
            'produit',
            [
                'id' => $prod['produit_id'],
                'produit_nom' => Tools::parseURL($prod['produit_nom_nom'])
            ]
        );
        if ($goodUrl != $this->request->getPathInfo()) {
            if ($this->request->getQueryString() != null) {
                $goodUrl.= '?' . $this->request->getQueryString();
            }
            return $this->redirect($goodUrl, 301);
        }

        // Fil d'arianne
        $categorieSpec->buildBreadcrumb($prod['categorie_id']);
        $bd->add($prod['produit_nom_nom']);

        $this->template->assign('p', $prod);

        // Images
        $arrImage = $produit->getImage(array('produit_id' => $prod['produit_id']));
        $this->template->assign('arrImage', $arrImage);

        // Caractéristiques
        $arrCaracteristique = $caracteristique->getCaraProduit(array('produit_id' => $prod['produit_id']));
        $this->template->assign('arrCaracteristique', $arrCaracteristique);

        // Attribut
        $attributs = $attribut->get(array('produit_id' => $prod['produit_id']));
        $this->template->assign('attributs', $attributs);

        // TODO : pré selectionner le disponible en premier, en réappro ensuite

        // Produits associes
        try {
            $configProduit = Boutique::getInstance()->getConfig('produit');
            $limitProdAssoc = $configProduit['associe'];
        } catch (\Exception $e) {
            $limitProdAssoc = 4;
        }
        $arrAssoc = $produit->getAssoc(array('produit_id' => $prod['produit_id'], 'limit' => $limitProdAssoc));
        if (count($arrAssoc) < $limitProdAssoc) {

            $tmp = $produit->getSearch([
                "categorie_id" => $prod['categorie_id'],
                "produit_id_not_id" => $prod['produit_id'],
                "limit" => $limitProdAssoc - count($arrAssoc),
                'order' => 'random'
            ]);

            $arrAssoc = array_merge($arrAssoc, $tmp);
        }
        $this->template->assign('arrAssoc', $arrAssoc);

        // Referencement
        $cat = new Categorie($prod["categorie_id"]);
        $tags = array_merge($prod, $cat->toArray());

        $ref['title'] = (!empty($prod['produit_nom_meta_title'])) ?
            $prod['produit_nom_meta_title'] : $txt->getOne('ref_title_produit', true, $tags);

        $ref['description'] = (!empty($prod['produit_nom_meta_desc'])) ?
            $prod['produit_nom_meta_desc'] : $txt->getOne('ref_desc_produit', true, $tags);

        $ref['keywords'] = (!empty($prod['produit_nom_meta_kw'])) ?
            $prod['produit_nom_meta_kw'] : $txt->getOne('ref_kw_produit', true, $tags);

        $ref['other'] = (!empty($prod['produit_nom_meta_other'])) ?
            $prod['produit_nom_meta_other'] : $txt->getOne('ref_other_produit', true, $tags);

        $ref['baseline'] = $txt->getOne('baseline_produit', true, $tags);
        $this->template->assign('ref', $ref);

        return $this->render('product.tpl');
    }

    /**
     * @Route(
     *      path         = "/product/select-option",
     *      name         = "select_option",
     *      methods      = { "POST" }
     * )
     *
     */
    public function selectOption()
    {
        $produitId = $this->request->get('produit_id');
        $produitAttributId = $this->request->get('produit_attribut_id');
        $produit = new Produit();
        $attribut = new Attribut();
        $json = array();

        if (!empty($produitAttributId)) {
            $arrProduitAttribut = $produit->getOne(array('produit_id' => $produitId, 'produit_attribut_id' => $produitAttributId));
            if (!empty($arrProduitAttribut)) {
                $attributs = $attribut->get(array('produit_id' => $produitId));
                $this->template->assign('attributs', $attributs);
                $this->template->assign('p', $arrProduitAttribut);
                $json['html'] = $this->template->fetch('product/addCart.tpl');
            }
        }

        return $this->response($json);
    }

    /**
     * @Route(
     *      path         = "/product/select-quantity",
     *      name         = "select_quantity",
     *      methods      = { "POST" }
     * )
     *
     */
    public function selectQuantity()
    {

        $json = array();

        $attribut = new Attribut();
        $result = $attribut->get([
            'produit_id' => $this->request->get('produit_id'),
            'produit_attribut_conditionnement' => array($this->request->get('quantity')),
            'order' => 'conditionnement'
        ]);

        if (!empty($result)) {
            $json['att_id'] = $result[0]['produit_attribut_id'];
            $json['total-remise'] = $result[0]['attribut_prix_ttc'] * $this->request->get('quantity');
            $json['total'] = $result[0]['attribut_produit_prix_ttc'] * $this->request->get('quantity');

            $devise = Boutique::getInstance()->getDevise();
            $json['total-remise'] = $devise->format($json['total-remise']);
            $json['total'] = $devise->format($json['total']);
        }

        return $this->response($json);
    }
}
