<?php

/**
 * Class Promotion
 */
class Promotion extends Model {

    /**
     * @var string
     */
    protected $table = 'promotion';

    /**
     *
     */
    function __construct() {

        parent::__construct();

        if (!isset($_SESSION['nb_promotion_panier'])) {
            $_SESSION['nb_promotion_panier'] = 0;
        }
    }

    /**
     * @param $promotion_code
     * @return bool
     */
    function initPromo($promotion_code) {

        // on efface la derniere promo
        $this->deletePromo();
        // on verifie si le code promo est valide
        $retour = $this->apply($promotion_code);
        if (!$retour)
            return false;
        else {
            $_SESSION['panier']['promotion_code'] = $promotion_code;
            $_SESSION['panier']['promotion_id'] = $retour;
            // fdp gratuit
            $_SESSION['panier']['promotion_fdp'] = $this->getFDPGratuit($retour);
            return true;
        }
    }

    /**
     *
     */
    function deletePromo() {

        foreach ($_SESSION['caddie'] as $key => $value) {
            if ($key > $_SESSION['nb_produit_panier']) {
                unset($_SESSION['caddie'][$key]);
            }
        }

        $_SESSION['nb_promotion_panier'] = 0;
        unset($_SESSION['panier']['promotion_code']);
        unset($_SESSION['panier']['promotion_id']);
        unset($_SESSION['panier']['promotion_fdp']);
    }

    /**
     * @param array $formvars
     * @param string $champs
     * @return array|null
     */
    function get($formvars = array(), $champs = "") {

        $sql = "SELECT *";
        $sql.=" FROM promotion p";
        if(empty($formvars['no_boutique'])) {
            $sql.=" INNER JOIN promotion_boutique pb ON (p.promotion_id = pb.promotion_id AND pb.boutique_id = '".Boutique::getInstance()->id."')";
        }
        $sql.=" WHERE 1";
        $sql.=" AND (promotion_nombre_utilisation>0 OR promotion_nombre_utilisation=-1)";
        $sql.=" AND promotion_statut_id = 1";
        if(empty($formvars['no_boutique'])) {
            $sql.=" AND pb.promotion_boutique_statut_id = 1";
        }
        $sql.=" AND promotion_date_debut <= NOW()";
        $sql.=" AND promotion_date_fin >= NOW()";

        if (!empty($formvars['promotion_code']))
            $sql.=" AND promotion_code = '" . $formvars['promotion_code'] . "'";

        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }

    /**
     * @param $promotionId
     *
     * permet de decrementer le nombre d'utilisation a chaque commande
     */
    function usePromo($promotionId) {
        $sql = "UPDATE promotion SET promotion_nombre_utilisation = promotion_nombre_utilisation - 1 WHERE promotion_id = '" . $promotionId . "' AND promotion_nombre_utilisation > 0";
        $this->sql->query($sql);
        // on supprime les promos du panier
        $this->deletePromo();
    }

    /**
     * @param $promoID
     * @return array
     */
    function getProduitByCategorie($promoID) {

        // niveau 1
        $sql = "SELECT p.produit_id, prc.promotion_categorie_inclus";
        $sql.=" FROM promotion_categorie prc, produit_categorie pc , produit p";
        $sql.=" WHERE prc.categorie_id = pc.categorie_id";
        $sql.=" AND pc.produit_id = p.produit_id";
        $sql.=" AND p.produit_statut_id = 1";
        $sql.=" AND promotion_id = '" . $promoID . "'";
        $sql.=" GROUP BY produit_id";
        $niv1 = $this->sql->getAll($sql);

        // niveau 2
        $sql = "SELECT p.produit_id, prc.promotion_categorie_inclus";
        $sql.=" FROM promotion_categorie prc, categorie c, produit_categorie pc , produit p";
        $sql.=" WHERE prc.categorie_id = c.parent_id";
        $sql.=" AND c.categorie_id = pc.categorie_id";
        $sql.=" AND pc.produit_id = p.produit_id";
        $sql.=" AND p.produit_statut_id = 1";
        $sql.=" AND promotion_id = '" . $promoID . "'";
        $sql.=" GROUP BY produit_id";
        $niv2 = $this->sql->getAll($sql);

        $allProd = array_merge($niv1, $niv2);

        return $allProd;
    }

    /**
     * @param $promoID
     * @return array
     */
    function getProduitByFournisseur($promoID) {

        // niveau 1
        $sql = "SELECT p.produit_id, pf.promotion_fournisseur_inclus";
        $sql.=" FROM promotion_fournisseur pf, produit p";
        $sql.=" WHERE pf.fournisseur_id = p.fournisseur_id";
        $sql.=" AND p.produit_statut_id = 1";
        $sql.=" AND promotion_id = '" . $promoID . "'";
        $sql.=" GROUP BY produit_id";
        return $this->sql->getAll($sql);
    }

    /**
     * @param $promoID
     * @return array
     */
    function getProduitByMarque($promoID) {

        // niveau 1
        $sql = "SELECT p.produit_id, pm.promotion_marque_inclus";
        $sql.=" FROM promotion_marque pm, produit p";
        $sql.=" WHERE pm.marque_id = p.marque_id";
        $sql.=" AND p.produit_statut_id = 1";
        $sql.=" AND promotion_id = '" . $promoID . "'";
        $sql.=" GROUP BY produit_id";
        return $this->sql->getAll($sql);
    }

    /**
     * @param $promoID
     * @return array
     */
    function getProduitByProduit($promoID) {

        // niveau 1
        $sql = "SELECT p.produit_id, pp.promotion_produit_inclus";
        $sql.=" FROM promotion_produit pp, produit p";
        $sql.=" WHERE pp.produit_id = p.produit_id";
        $sql.=" AND p.produit_statut_id = 1";
        $sql.=" AND promotion_id = '" . $promoID . "'";
        $sql.=" GROUP BY produit_id";
        return $this->sql->getAll($sql);
    }

    /**
     * @param $promoID
     * @return array
     */
    function getFDPGratuit($promoID) {

        $sql = "SELECT livraison_id, zone_id";
        $sql.=" FROM promotion_fdp";
        $sql.=" WHERE  promotion_id = '" . $promoID . "'";
        $sql.=" GROUP BY promotion_fdp_id";
        return $this->sql->getAll($sql);
    }

    /**
     * @param $arrPort
     */
    function validFDPGratuit(&$arrPort) {

        // si pas d'offre gratuite
        if (empty($_SESSION['panier']['promotion_fdp']))
            return false;

        foreach ($arrPort as &$port) {
            $gratos = false;
            foreach ($_SESSION['panier']['promotion_fdp'] as $fdp) {
                if ($fdp['livraison_id'] == $port['livraison_id'] && $fdp['zone_id'] == $port['zone_id']) {
                    $gratos = true;
                    break;
                }
            }
            if ($gratos == true) {
                $port['port_ht'] = 0;
                $port['port_ttc'] = 0;
            }
        }
    }

    /**
     * on regarde les promotions applicables
     *
     * @param $promoCode
     *
     * @return mixed
     * @throws DediException
     */

    function apply($promoCode) {

        if (empty($promoCode))
            return false;

        $zePromo = $this->get(array('promotion_code' => $promoCode));
        // si pas de promo
        if (empty($zePromo)) {
            $texte = new Texte();
            throw new DediException($texte->getOneOrCreate('noCodePromo', 'Ce code promo n\'existe pas.'));
        }

        $zePromo = $zePromo[0];

        $panier = Panier::getInstance();

        // unique or sum type testing
        //$canApply = $panier->canApplyType($zePromo["promotion_cumul"]);
        /*if($canApply == false) {
            $texte = new Texte();
            throw new DediException($texte->getOneOrCreate('errPromoType',
                'Ces types de promotions ne sont pas cumulables et ne peuvent pas être utilisés ensemble.'));
        }*/

        $reduction = new Reduction();
        $reduction->code = $zePromo['promotion_code'];
        if ($panier->haveReduction($reduction) == true) {
            $texte = new Texte();
            throw new DediException($texte->getOneOrCreate('errPromoInCart',
                'Cette promotion est déjà comprise dans votre panier.'));
        }

        // on check le minimum d'achat
        list($totalHT, $totalTTC) = $panier->getTotalProducts();

        if ($totalHT < $zePromo['promotion_min_achat']) {
            $texte = new Texte();
            $minAchat = Boutique::getInstance()->getDevise()->format($zePromo['promotion_min_achat']);
            throw new DediException($texte->getOneOrCreate('codePromoMinAchat', 'Le minimum d\'achat pour ce code promo est de ' . $minAchat));
        }

        // on check si le client peut encore utiliser cette promo
        if ($zePromo['promotion_nombre_utilisation_client'] != -1) {
            $customer = $panier->getCustomer();
            if($customer instanceof Client) {
                $countPromoClientUsed = $this->getPromotionClient($zePromo['promotion_id'], $customer->client_id);
                if ($zePromo['promotion_nombre_utilisation_client'] <= $countPromoClientUsed) {
                    $texte = new Texte();
                    throw new DediException($texte->getOneOrCreate('errPromoUsedMax',
                        'Vous ne pouvez plus utiliser cette promotion.'));
                }
            }
        }

        // on check le minimum de qte
        $qteArticle = $panier->getCountProducts();

        if ($qteArticle < $zePromo['promotion_min_qte']) {
            $texte = new Texte();
            throw new DediException($texte->getOneOrCreate('codePromoMinQte', 'Vous n\'avez pas assez de produits dans votre panier pour utiliser ce code promo.'));
        }

        $includeProd = array();
        $excludeProd = array();

        // on gere le cas "tous les produits"
        if ($zePromo['promotion_all'] == 1) {
            $sql = "SELECT produit_id FROM produit WHERE produit_statut_id = 1";
            $arrProd = $this->sql->getAll($sql);
            foreach ($arrProd as $prod) {
                array_push($includeProd, $prod['produit_id']);
            }
        }

        // on gere les categories
        $arrProd = $this->getProduitByCategorie($zePromo['promotion_id']);
        if (!empty($arrProd)) {
            foreach ($arrProd as $prod) {
                if ($prod['promotion_categorie_inclus'] == 1)
                    array_push($includeProd, $prod['produit_id']);
                else
                    array_push($excludeProd, $prod['produit_id']);
            }
        }

        // on gere les fournisseurs
        $arrProd = $this->getProduitByFournisseur($zePromo['promotion_id']);
        if (!empty($arrProd)) {
            foreach ($arrProd as $prod) {
                if ($prod['promotion_fournisseur_inclus'] == 1)
                    array_push($includeProd, $prod['produit_id']);
                else
                    array_push($excludeProd, $prod['produit_id']);
            }
        }

        // on gere les marques
        $arrProd = $this->getProduitByMarque($zePromo['promotion_id']);
        if (!empty($arrProd)) {
            foreach ($arrProd as $prod) {
                if ($prod['promotion_marque_inclus'] == 1)
                    array_push($includeProd, $prod['produit_id']);
                else
                    array_push($excludeProd, $prod['produit_id']);
            }
        }

        // on gere les produits
        $arrProd = $this->getProduitByProduit($zePromo['promotion_id']);
        if (!empty($arrProd)) {
            foreach ($arrProd as $prod) {
                if ($prod['promotion_produit_inclus'] == 1)
                    array_push($includeProd, $prod['produit_id']);
                else
                    array_push($excludeProd, $prod['produit_id']);
            }
        }

        $includeProd = array_values(array_unique($includeProd));
        $excludeProd = array_values(array_unique($excludeProd));

        $arrPromo = array_diff($includeProd, $excludeProd);

        if (empty($arrPromo)) {
            $texte = new Texte();
            throw new DediException($texte->getOneOrCreate('codePromoNoApplication', 'Ce code promo n\'est pas applicable'));
        }

        $arrPanier = $panier->getProducts();

        $listPromo = array();

        foreach ($arrPanier as &$prod) {

            // si le produit n'est pas concerne on zap
            if (!in_array($prod['produit_id'], $arrPromo))
                continue;

            // si prix barre, on zap
            /*if ($prod['prix_barre'] && $zePromo['promotion_prix_barre'] == 0)
                continue;*/

            $prodPrixTtc = round($prod['produit_prix_ht'] * (1 + $prod['taxe_taux']), 4);

            if ($zePromo['promotion_prefix_id'] == 1) {
                $promo = $zePromo['promotion_prix'];
            } else {
                $promo = $prodPrixTtc * ($zePromo['promotion_prix'] / 100);
            }

            if (!empty($promo) && $promo > $prod['produit_prix_ht']) {
                $promo = $prodPrixTtc;
            }

            if ($promo > 0) {
                $newPromo = array();
                $newPromo['taxe_taux'] = $prod['taxe_taux'];
                $newPromo['produit_prix'] = $promo;

                if ($zePromo['promotion_application'] == 0) {
                    $newPromo['produit_prix'] = $promo * $prod['quantite_order'];
                }

                array_push($listPromo, $newPromo);
            }
        }

        $data = array(
            'id' => $zePromo['promotion_id'],
            'name' => $zePromo['promotion_nom'],
            'code' => $zePromo['promotion_code'],
            'price' => 0,
            'taxe' => 0
            //'cumul' => $zePromo["promotion_cumul"],
        );
        // on affecte la promo selon le champ d'application
        switch ($zePromo['promotion_application']) {
            // tous les produits
            case 0:
                foreach ($listPromo as $p) {

                    // si promotion par valeur, on regarde que l'on depasse pas
                    if ($zePromo['promotion_prefix_id'] == 1) {
                        if ($data['price'] >= $zePromo['promotion_prix']) {
                            continue;
                        } else {
                            if (($data['price'] + $p['produit_prix']) > $zePromo['promotion_prix']) {
                                $priceReduction = $zePromo['promotion_prix'] - $data['produit_prix'];
                                $data['price'] += $priceReduction;
                                $data['taxe'] += $priceReduction - $priceReduction / (1 + $p['taxe_taux']);
                            } else {
                                $data['price'] += $zePromo['promotion_prix'];
                                $data['taxe'] += $zePromo['promotion_prix'] * $p['taxe_taux'];
                            }
                        }
                    } else {
                        $data['price'] += $p['produit_prix'];
                        $data['taxe'] += $p['produit_prix'] - $p['produit_prix'] / (1 + $p['taxe_taux']);
                    }
                }
                break;
            // le plus cher
            case 1:
                foreach ($listPromo as $p) {
                    if (!isset($data['produit_prix']) || $data['produit_prix'] < $p['produit_prix']) {
                        $data['produit_prix'] = $p['produit_prix'];
                        $data['price'] = $p['produit_prix'];
                        $data['taxe'] = $data['price'] * $p['taxe_taux'];
                    }
                }
                break;
            // le moins cher
            case 2:
                foreach ($listPromo as $p) {
                    if (!isset($data['produit_prix']) || $data['produit_prix'] > $p['produit_prix']) {
                        $data['produit_prix'] = $p['produit_prix'];
                        $data['price'] = $p['produit_prix'];
                        $data['taxe'] = $data['price'] * $p['taxe_taux'];
                    }
                }
                break;
        }

        if (!empty($data['price'])) {
            $data['quantity'] = 1;
            unset($data['produit_prix']);
            $reduction = new Reduction();
            $reduction->hydrate($data);

            return $reduction;
        }

        return false;
    }

    public function checkReduction() {
        $panier = Panier::getInstance();
        $reduction = $panier->getReduction();

        if (!empty($reduction)) {
            $panier->initReduction();
            try{
                $reduc = $this->apply($reduction->code);
                if ($reduc instanceof Reduction) {
                    $panier->addReduction($reduc);
                }
            } catch (DediException $e) {

            }
        }
    }

    public function getPromotionClient($promoID, $client_id)
    {
        $sql = "SELECT p.promotion_client_id as nbutil";
        $sql .= " FROM promotion_client p";
        $sql .= " WHERE p.client_id = '" . $client_id . "'";
        $sql .= " AND promotion_id = '" . $promoID . "'";

        $arrPromoClient = $this->sql->getAll($sql);
        return count($arrPromoClient);
    }
}