<?php

use Symfony\Component\Routing\Exception\InvalidParameterException;
use Tree\Node;
use Tree\Tree;

/**
 * @property int $selection_id
 */
abstract class Selection extends Model
{
    protected $table = "selection";

    /** @var string */
    protected $type;

    static protected $tree = null;

    public function __construct($id = null)
    {
        $this->innerJoin[] = [
            "selection_boutique sb",
            "AND",
            ["sb.selection_id", "s.selection_id"],
            ["sb.boutique_id", Boutique::getInstance()->id]
        ];

        $this->where[] = "s.selection_type = '{$this->type}'";

        parent::__construct($id);
    }

    abstract protected static function newSelf(int $id = null) : Selection;

    /**
     * @param $selectionId
     * @param bool $noBoutique
     * @return CategorieSpec[]
     * @throws Exception
     */
    public static function getBranch($selectionId, $noBoutique = false)
    {
        $treeSelection = [];
        do {
            $cat = static::newSelf($selectionId);

            if($noBoutique || $cat->selection_boutique_statut_id == 1){
                $treeSelection[] = $cat;
            }

            $selectionId = $cat->parent_id;
        } while (!empty($selectionId));

        $treeSelection = array_reverse($treeSelection);
        return $treeSelection;
    }

    /**
     * Retourne un arbre avec toutes les categories du site
     *
     * @param array $formvars
     * @return bool|null|Node|Tree
     * @throws Exception
     */
    public static function getTree($formvars = array())
    {
        if (is_null(self::$tree)) {
            self::$tree = new Tree();

            $selection = static::newSelf();

            // Récupération de toutes les selections en 1 requete
            $selections = $selection->get();

            // Tri des selections par niveaux
            foreach ($selections as $k => $select) {
                $selections[$k]['depth'] = substr_count($select['selection_ext_id'], ',') + 1;
                $catDepth[$k] = $selections[$k]['depth'];
            }

            array_multisort($catDepth, SORT_ASC, $selections);

            foreach ($selections as $sel) {
                if (empty($sel['parent_id'])) {
                    self::$tree->addNode(new Node($sel));
                } else {
                    $parent = self::$tree->findNode(array('selection_id' => $sel['parent_id']));
                    if ($parent instanceof Node) {
                        $parent->addChildren(new Node($sel));
                    }
                }
            }
        }
        if (!empty($formvars)) {
            return self::$tree->findNode($formvars);
        } else {
            return self::$tree;
        }
    }

    /**
     * @param array $treeSelection
     * @param int $k index of the categorie for witch you want to build the url
     *                              If not precised, the last categorie is used
     * @return string
     * @internal param $tc
     * @internal param $router
     */
    public static function generateUrlFromBranch(array $treeSelection, $k = null)
    {
        if (is_null($k)) {
            $k = count($treeSelection) - 1;
        }
        $router = Application::getRouter()->getRouter();

        $paramUrl = array();
        $paramUrl['categorie_nom'] = Tools::parseURL($treeSelection[$k]->selection_nom_nom);
        $paramUrl['id'] = $treeSelection[$k]->selection_id;
        if (!empty($treeSelection[$k - 1])) {
            $paramUrl['parent_nom'] = Tools::parseURL($treeSelection[$k - 1]->selection_nom_nom);
        }
        if (!empty($treeSelection[$k - 2])) {
            $paramUrl['grand_parent_nom'] = Tools::parseURL($treeSelection[$k - 2]->selection_nom_nom);
        }

        try {
            $url = $router->generate('liste_' . ($k + 1), $paramUrl);
        } catch (InvalidParameterException $e) {
            $debugBar = Application::getDebugBar();
            if (!empty($debugBar)) {
                $debugBar['exceptions']->addException($e);
            }
            $url = '';
        }
        return $url;
    }

    /**
     * @param $selectionId
     * @throws Exception
     */
    public static function buildBreadcrumb($selectionId)
    {
        $bd = Breadcrumb::getInstance();
        $treeSelection = self::getBranch($selectionId);
        /**
         * @var int $k
         * @var Selection $tc */
        foreach ($treeSelection as $k => $tc) {
            $bd->add($tc->selection_nom_nom, self::generateUrlFromBranch($treeSelection, $k));
        }
    }

    /**
     * Mise a jour des identifants etendus par categorie
     *
     * @param $selection_id
     * @throws DediException
     * @throws Exception
     */
    public function updateExtId($selection_id)
    {
        $selection_ext_id = $selection_id;
        $arrSelection[0]['parent_id'] = $selection_id;
        do {

            $arrSelection = $this->get(['s.selection_id' => $arrSelection[0]['parent_id']]);
            if (!empty($arrSelection[0]['parent_id'])) {
                $selection_ext_id .= "," . $arrSelection[0]['parent_id'];
            }
        } while (!empty($arrSelection[0]['parent_id']));

        $toSave = [
            "selection_id" => $selection_id,
            "selection_ext_id" => $selection_ext_id,
        ];

        $this->save($toSave);

        // on met a jour les enfants
        $sql = "SELECT selection_Id FROM selection WHERE parent_id = '" . $selection_id . "'";
        $this->sql->query($sql, SiteSQL::SQL_ALL, SiteSQL::SQL_ASSOC);
        $arrEnfant = $this->sql->record;
        if (!empty($arrEnfant)) {
            foreach ($arrEnfant as $enfant) {
                $this->updateExtId($enfant['selection_id']);
            }
        }
    }

}